import * as d3 from 'd3';

export const graph = (
  selection,
  { nodes, links, transform, setState },
) => {
  // Define the arrowhead marker
  selection
    .append('marker')
    .attr('id', 'arrowhead')
    .attr('markerWidth', 10)
    .attr('markerHeight', 10)
    .attr('refX', 0)
    .attr('refY', 3)
    .attr('orient', 'auto')
    .append('path')
    .attr('d', 'M0,0 L0,6 L9,3 z') // Path for arrowhead shape
    .style('fill', '#000'); // Arrowhead color

  //thank ChatGPT for the line ends code
  selection
    .selectAll('path.edges')
    .data(links)
    .join('path')
    .attr('class', 'edges')
    .attr('d', (d) => {
      const source = nodes.find(
        (node) => node.id === d.patient_id,
      );
      const target = nodes.find(
        (node) => node.id === d.department_referral,
      );
      if (!source || !target) return '';
      const dx = target.x - source.x;
      const dy = target.y - source.y;
      const angle = Math.atan2(dy, dx); // Angle of the line segment
      const controlPoint = {
        x:
          source.x +
          dx / 2 +
          5 * Math.cos(angle + Math.PI / 2),
        y:
          source.y +
          dy / 2 +
          5 * Math.sin(angle + Math.PI / 2),
      }; // Control point 10 units above the line
      return `M${source.x},${source.y} Q${controlPoint.x},${controlPoint.y} ${target.x},${target.y} ${target.x - 10},${target.y - 10}`; // Quadratic Bézier curve path
    })
    .attr('fill', 'none')
    .attr('stroke', '#000')
    .attr('stroke-width', 1)
    .attr('marker-end', 'url(#arrowhead)');

  selection
    .selectAll('circle.nodes')
    .data(nodes)
    .join('circle')
    .attr('class', 'nodes')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y)
    .attr('r', (d) => d.r)
    .attr('fill', (d) => d.fill)
    .attr('opacity', 0.5);

  selection
    .selectAll('text.nodes')
    .data(nodes)
    .join('text')
    .attr('class', 'nodes')
    .attr('x', (d) => d.x)
    .attr('y', (d) => d.y)
    .text((d) => d.id)
    .attr('text-anchor', 'middle')
    .attr('dominant-baseline', 'middle');
};