import * as d3 from 'd3';
import { graph } from './graph';

export const main = (container, { state, setState }) => {
  const width = container.clientWidth;
  const height = container.clientHeight;

  const svg = d3
    .select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#F0FFF4');

  // Zoom code
  const { transform } = state;
  const zoomBehavior = d3.zoom().on('zoom', (event) => {
    setState((state) => ({
      ...state,
      transform: event.transform,
    }));
  });
  svg.call(zoomBehavior);

  const graphGroup = svg
    .selectAll('g.graph')
    .data([null])
    .join('g')
    .attr('class', 'graph');

  graphGroup.attr('transform', transform);

  // Data
  const data = [
    { id: 'node1', x: 50, y: 50, r: 10, fill: 'red' },
    { id: 'node2', x: 100, y: 100, r: 15, fill: 'blue' },
    { id: 'node3', x: 150, y: 150, r: 20, fill: 'green' },
  ];

  const links = [
    { source: data[0], target: data[1] },
    { source: data[1], target: data[2] },
  ];

  // Simulation
  const sim = d3
    .forceSimulation(data)
    .force(
      'charge',
      d3
        .forceManyBody()
        .strength(-30)
        .distanceMax((d) => d.r + 15),
    )
    .force(
      'collide',
      d3.forceCollide().radius((d) => d.r + 1),
    )
    .force(
      'link',
      d3
        .forceLink(links)
        .id((d) => d.id)
        .distance(60),
    )
    .force(
      'center',
      d3.forceCenter(width / 2, height / 2).strength(0.5),
    )
    .on('tick', () =>
      graph(graphGroup, {
        nodes: data,
        links,
        transform,
        setState,
      }),
    );
};
